#!/bin/bash
### 1. BE SURE TO LOAD THE MPICH2 MODULE FIRST!
### module load mpich2
### mpdtrace -l
### These variables control the naming of IDL data files
### 2. give mdiv_patch.sql & load_patch.awk the apropriate number of fields
### module load mpich2
table=mdiv_patch
#PCA#tmp_table=mdiv_patch_tmp
tmax=60
#CLASS#cname='Gastropoda'
#PCA#fname1='~/idl/mdiv/data/mdiv_patch_tmp.dat'
fname1=~/clust/k-means/patch/obs.raw.patch
fname2=~/idl/mdiv/data/mdiv_patch.dat
fname4=~/idl/mdiv/out/mdiv_0-60.dat
fname3=~/idl/mdiv/data/gnames.dat
#CLASS#fname3='~/idl/mdiv/data/cnames.dat'
###
echo `date`
pushd ~/clust/k-means/patch/
echo Clean-up clustering files
rm -f obs.raw* obs.bin* seeds.out.*
popd
### Create database sed_rock
createdb sed_rock
echo Creating table mdiv_lith, and mdiv_env, then table mdiv, which depends on mdiv_lith and mdiv_env.
psql sed_rock < mdiv_lith.sql
psql sed_rock < mdiv_env.sql
psql sed_rock < mdiv.sql
psql sed_rock < mdiv_meso.sql
echo Done
echo ""
### Fix up the .csv file and extract the desired fields
echo Parsing the initial data.
./fix_fields.bash | ./extract_fields.bash > mdiv.db
echo Done
echo ""
### Create SQL commands and load the fields into tables mdiv_lith and mdiv
echo Populating table mdiv_lith.
awk -f load_lith.awk mdiv_lith.db | psql sed_rock
echo Done populating table mdiv_lith.
echo ""
echo Populating table mdiv_env.
awk -f load_env.awk mdiv_env.db | psql sed_rock
echo Done populating table mdiv_env.
echo ""
echo Populating table mdiv.
awk -f load.awk mdiv.db | psql sed_rock
echo Done populating table mdiv.
echo ""
### Check number of records in mdiv table
echo "Verifing table mdiv. The number of entries in mdiv is: `psql sed_rock -tc 'select count(*) from mdiv'`"
### and compare with the extracted data file
echo "The number of entries above should match this number: `wc -l mdiv.db | awk '{print $1}'`"
echo ""
### Add PostGIS geometry column 'geom' into table mdiv under schema
### 'public'. 'geom' will hold a point with 2 dimensions and has using
### SRID 4326, which corresponds to WGS84 datum.
### Ensure PostGIS is installed and that you have permissions to add and 
### Delete from column geom_columns. What follows here are the complete 
### instructions for enabling PostGIS functions under Fedora.
### # yum install postgis
###$ createlang plpgsql sed_rock
###$ cd /usr/share/pgsql/contrib
###$ psql sed_rock -f lwpostgis.sql
###$ psql sed_rock -f spatial_ref_sys.sql
###     If you are able to execute these commands on your database, then 
###	you already own tables geom_columns and spatial_ref_sys. If not 
###	you must have user postgres perform the steps above, then transfer 
###	ownership of the two tables using the following:
###sed_rock=# alter table geometry_columns owner to user_name;
###sed_rock=# alter table spatial_ref_sys owner to user_name;
###
echo Creating and populating spatial column 'geom' to table mdiv
psql sed_rock --command "SELECT AddGeometryColumn ('public', 'mdiv', 'geom', 4326, 'POINT', 2)"
### Populate 'geom' with the longitude,latitude data
psql sed_rock --command "UPDATE mdiv SET geom = PointFromText('POINT(' || londec || ' ' || latdec || ')', 4326)"
echo Done
echo ""
### Query table mdiv for clustering fields and to generate table mdiv_clusters,
### which will also include the cluster assingment for each observaiton
echo Done
echo ""
echo Create ${table}_tmp for IDL PCA routine
### Use this to isolate the results to a specific taxonomic class
#CLASS#psql sed_rock -tc "select ma_mid,class_name,genus_name,species_name,mdiv.lithology,mdiv.environment,latdec,londec,geom,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and class_name='${cname}' and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1" | head -n-1 | sed 's/^ * //g;s/ * | * /|/g;s/| * /|/g;s/ * |/|/g' > ${table}_tmp.db
#PCA#psql sed_rock -tc "select ma_mid,class_name,genus_name,species_name,mdiv.lithology,mdiv.environment,latdec,londec,geom,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1" | head -n-1 | sed 's/^ * //g;s/ * | * /|/g;s/| * /|/g;s/ * |/|/g' > ${table}_tmp.db
#PCA#psql sed_rock < ${table}_tmp.sql
#PCA#awk -f load_patch_tmp.awk ${table}_tmp.db | psql sed_rock
#PCA#echo Parsing output for PCA routine
#PCA#echo "select ma_mid,latdec,londec,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from ${table}_tmp" | psql sed_rock -t | awk -F'|' '{printf "%s  %s  %s  %s  %s  %s  %s  %s\n", $1,$2,$3,$4,$5,$6,$7,$8;}' | head -n-1 > ${fname1}
#PCA#echo Starting PCA routine
#PCA#pushd ~/idl/mdiv
#PCA#usr/local/itt/idl64/bin/idl << EOF
#PCA#pca
#PCA#exit
#PCA#EOF
#PCA#popd
#PCA#echo Done
#PCA#echo ""
###
### Run the k-means clustering algorithm for all cluster sizes, 
### then parse the cluster assignments and within cluster variance 
### from the clustering runs, which will be used in a scree plot. This 
### will be done later.
echo Parse raw data for clustering
psql sed_rock -tc "select pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1" | head -n-1  | sed 's/|//g' > ${fname1}
echo 'Starting k-means clustering for n=5...100 clusters'
pushd ~/clust/k-means/patch/
./loop.bash
#echo 'Starting k-means clustering for n=40 clusters, trials=1...3'
#./loop_40.bash
popd
echo Appending cluster assignments to PSQL table ${table}
echo Parsing in cluster assignments for n=5...100 clusters
psql sed_rock -tc "select ma_mid,class_name,genus_name,species_name,mdiv.lithology,mdiv.environment,latdec,londec,geom,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth from mdiv,mdiv_lith,mdiv_env where ma_mid<${tmax} and mdiv.lithology=mdiv_lith.lithology and mdiv.environment=mdiv_env.environment and mdiv.lithology is not null and mdiv.environment is not null and depth > -1" | head -n-1 | sed 's/^ * //g;s/ * | * /|/g;s/| * /|/g;s/ * |/|/g' | paste -d'|' - ~/clust/k-means/patch/clusters.out.patch.5 > ${table}.db
#
paste -d'|' ${table}.db ~/clust/k-means/patch/clusters.out.patch.10 > ${table}.db1; mv ${table}.db1 ${table}.db
#
for ((i=20;i<=100;i+=20)); do
   paste -d'|' ${table}.db ~/clust/k-means/patch/clusters.out.patch.$i > ${table}.db1; mv ${table}.db1 ${table}.db
done
### Parse in cluster assignments for random seed clustering trials
### for n=40 clusters
#for ((i=1;i<=3;i++)); do
#   paste -d'|' ${table}.db ~/clust/k-means/patch/clusters.out.patch.40.$i > ${table}.db1; mv ${table}.db1 ${table}.db
#done
#
echo Create ${table} for IDL routines
psql sed_rock < ${table}.sql
awk -f load_patch.awk ${table}.db | psql sed_rock
echo Done
echo ""
echo Parsing queried data before generating multivariate IDL maps
echo Data 0-${tmax} Ma
echo "select ma_mid,latdec,londec,pct_sand,pct_silt,pct_clay,pct_lime_mud,depth,clust_num_10,clust_num_20,clust_num_40,clust_num_60,clust_num_80,clust_num_100 from ${table} order by ma_mid" | psql sed_rock -t | awk -F'|' '{printf "%s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s  %s\n", $1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14;}' | head -n-1 > ${fname2}
#
echo "select genus_name from ${table} where ma_mid < ${tmax}" | psql sed_rock -t | head -n-1 | sed 's/^ * //' > ${fname3}
#CLASS#echo "select class_name from ${table} where ma_mid < ${tmax}" | psql sed_rock -t | head -n-1 | sed 's/^ * //' > ${fname3}
echo Done
echo ""
###
echo Finding mesoscale diversity and generating metacommunity diversity maps
pushd ~/idl/mdiv
/usr/local/itt/idl64/bin/idl << EOF
mdiv
count_map
multi_map
exit
EOF
popd
echo Done
echo ""
echo Parsing IDL output from mdiv.pro and loading into PSQL table mdiv_meso.
cat ${fname4} |  awk -F' ' '{printf "%s|%s|%s|%s|%s|%s|%s|%s\n", $1,$2,$3,$4,$5,$6,$7,$8;}' > mdiv_meso.db
echo Populating table mdiv_meso.
awk -f load_meso.awk mdiv_meso.db | psql sed_rock
echo Done populating table mdiv_meso
echo ""
echo All done 
echo `date`
#sed 's/|//g;s/^      //' | 
